@@ -1,4 +1,4 @@
-# Copyright (c) 2016-2021 by Ron Frederick <ronf@timeheart.net> and others.
+# Copyright (c) 2016-2024 by Ron Frederick <ronf@timeheart.net> and others.
 #
 # This program and the accompanying materials are made available under
 # the terms of the Eclipse Public License v2.0 which accompanies this
@@ -27,7 +27,7 @@ import asyncio
 import ctypes
 import ctypes.wintypes
 import errno
-from typing import TYPE_CHECKING, Optional, Tuple, Union, cast
+from typing import TYPE_CHECKING, Tuple, Union, cast
 
 from .misc import open_file
 
@@ -78,7 +78,7 @@ class _PageantTransport:
     """Transport to connect to Pageant agent on Windows"""
 
     def __init__(self) -> None:
-        self._mapname = '%s%08x' % (_AGENT_NAME, win32api.GetCurrentThreadId())
+        self._mapname = f'{_AGENT_NAME}{win32api.GetCurrentThreadId():08x}'
 
         try:
             self._mapfile = mmapfile.mmapfile('', self._mapname,
@@ -164,8 +164,7 @@ class _W10OpenSSHTransport:
         """Wait for the transport to close"""
 
 
-async def open_agent(agent_path: Optional[str]) -> \
-        Tuple['AgentReader', 'AgentWriter']:
+async def open_agent(agent_path: str) -> Tuple['AgentReader', 'AgentWriter']:
     """Open a connection to the Pageant or Windows 10 OpenSSH agent"""
 
     transport: Union[None, _PageantTransport, _W10OpenSSHTransport] = None
@@ -178,7 +177,6 @@ async def open_agent(agent_path: Optional[str]) -> \
             agent_path = _DEFAULT_OPENSSH_PATH
 
     if not transport:
-        assert agent_path is not None
         transport = _W10OpenSSHTransport(agent_path)
 
     return transport, transport
