@@ -1,4 +1,4 @@
-# Copyright (c) 2019-2021 by Ron Frederick <ronf@timeheart.net> and others.
+# Copyright (c) 2019-2024 by Ron Frederick <ronf@timeheart.net> and others.
 #
 # This program and the accompanying materials are made available under
 # the terms of the Eclipse Public License v2.0 which accompanies this
@@ -29,30 +29,34 @@ from .packet import Byte, MPInt, String, UInt32, SSHPacket
 from .public_key import KeyExportError, SSHKey, SSHOpenSSHCertificateV01
 from .public_key import register_public_key_alg, register_certificate_alg
 from .public_key import register_sk_alg
-from .sk import SSH_SK_ECDSA, SSH_SK_USER_PRESENCE_REQD, sk_enroll, sk_sign
+from .sk import SSH_SK_ECDSA, SSH_SK_USER_PRESENCE_REQD
+from .sk import sk_enroll, sk_sign, sk_webauthn_prefix, sk_use_webauthn
 
 
-_PrivateKeyArgs = Tuple[bytes, bytes, bytes, int, bytes, bytes]
-_PublicKeyArgs = Tuple[bytes, bytes, bytes]
+_PrivateKeyArgs = Tuple[bytes, bytes, str, int, bytes, bytes]
+_PublicKeyArgs = Tuple[bytes, bytes, str]
 
 
 class _SKECDSAKey(SSHKey):
-    """Handler for elliptic curve public key encryption"""
+    """Handler for U2F ECDSA public key encryption"""
 
     _key: ECDSAPublicKey
 
     use_executor = True
 
-    def __init__(self, curve_id: bytes, public_value: bytes,
-                 application: bytes, flags: int = 0,
-                 key_handle: bytes = None, reserved: bytes = b''):
+    def __init__(self, curve_id: bytes, public_value: bytes, application: str,
+                 flags: int = 0, key_handle: Optional[bytes] = None,
+                 reserved: bytes = b''):
         super().__init__(ECDSAPublicKey.construct(curve_id, public_value))
 
         self.algorithm = b'sk-ecdsa-sha2-' + curve_id + b'@openssh.com'
-        self.sig_algorithms = (self.algorithm,)
+        self.sig_algorithms = (self.algorithm, b'webauthn-' + self.algorithm)
         self.all_sig_algorithms = set(self.sig_algorithms)
 
+        self.use_webauthn = sk_use_webauthn
+
         self._application = application
+        self._app_hash = sha256(application.encode('utf-8')).digest()
         self._flags = flags
         self._key_handle = key_handle
         self._reserved = reserved
@@ -83,7 +87,6 @@ class _SKECDSAKey(SSHKey):
 
         # pylint: disable=arguments-differ
 
-        application = application.encode('utf-8')
         flags = SSH_SK_USER_PRESENCE_REQD if touch_required else 0
 
         public_value, key_handle = sk_enroll(SSH_SK_ECDSA, application,
@@ -117,7 +120,7 @@ class _SKECDSAKey(SSHKey):
 
         curve_id = packet.get_string()
         public_value = packet.get_string()
-        application = packet.get_string()
+        application = packet.get_string().decode('utf-8')
         flags = packet.get_byte()
         key_handle = packet.get_string()
         reserved = packet.get_string()
@@ -130,7 +133,7 @@ class _SKECDSAKey(SSHKey):
 
         curve_id = packet.get_string()
         public_value = packet.get_string()
-        application = packet.get_string()
+        application = packet.get_string().decode('utf-8')
 
         return curve_id, public_value, application
 
@@ -164,27 +167,46 @@ class _SKECDSAKey(SSHKey):
     def sign_ssh(self, data: bytes, sig_algorithm: bytes) -> bytes:
         """Compute an SSH-encoded signature of the specified data"""
 
-        # pylint: disable=unused-argument
-
         if self._key_handle is None:
             raise ValueError('Key handle needed for signing')
 
-        flags, counter, sig = sk_sign(sha256(data).digest(), self._application,
-                                      self._key_handle, self._flags)
+        is_webauthn = sig_algorithm.startswith(b'webauthn')
+
+        flags, counter, sig, client_data = sk_sign(data, self._application,
+                                                   self._key_handle,
+                                                   self._flags, is_webauthn)
 
         r, s = cast(Tuple[int, int], der_decode(sig))
 
-        return String(MPInt(r) + MPInt(s)) + Byte(flags) + UInt32(counter)
+        sig = String(MPInt(r) + MPInt(s)) + Byte(flags) + UInt32(counter)
+
+        if is_webauthn:
+            sig += String(self._application) + String(client_data) + String('')
+
+        return sig
 
     def verify_ssh(self, data: bytes, sig_algorithm: bytes,
                    packet: SSHPacket) -> bool:
         """Verify an SSH-encoded signature of the specified data"""
 
-        # pylint: disable=unused-argument
+        is_webauthn = sig_algorithm.startswith(b'webauthn')
 
         sig = packet.get_string()
         flags = packet.get_byte()
         counter = packet.get_uint32()
+
+        if is_webauthn:
+            _ = packet.get_string()            # origin
+            client_data = packet.get_string()
+            _ = packet.get_string()            # extensions
+
+            prefix = sk_webauthn_prefix(data, self._application)
+
+            if not client_data.startswith(prefix):
+                return False
+
+            data = client_data
+
         packet.check_end()
 
         if self._touch_required and not flags & SSH_SK_USER_PRESENCE_REQD:
@@ -197,9 +219,9 @@ class _SKECDSAKey(SSHKey):
 
         sig = der_encode((r, s))
 
-        return self._key.verify(sha256(self._application).digest() +
-                                Byte(flags) + UInt32(counter) +
-                                sha256(data).digest(), sig, 'sha256')
+        return self._key.verify(self._app_hash + Byte(flags) +
+                                UInt32(counter) + sha256(data).digest(),
+                                sig, 'sha256')
 
 
 _algorithm = b'sk-ecdsa-sha2-nistp256@openssh.com'
@@ -207,7 +229,8 @@ _cert_algorithm = b'sk-ecdsa-sha2-nistp256-cert-v01@openssh.com'
 
 register_sk_alg(SSH_SK_ECDSA, _SKECDSAKey, b'nistp256')
 
-register_public_key_alg(_algorithm, _SKECDSAKey, True, (_algorithm,))
+register_public_key_alg(_algorithm, _SKECDSAKey, True,
+                        (_algorithm, b'webauthn-' + _algorithm))
 
 register_certificate_alg(1, _algorithm, _cert_algorithm,
                          _SKECDSAKey, SSHOpenSSHCertificateV01, True)
