@@ -1,6 +1,6 @@
-#!/usr/bin/env python3.6
+#!/usr/bin/env python3.7
 #
-# Copyright (c) 2016-2021 by Ron Frederick <ronf@timeheart.net> and others.
+# Copyright (c) 2016-2024 by Ron Frederick <ronf@timeheart.net> and others.
 #
 # This program and the accompanying materials are made available under
 # the terms of the Eclipse Public License v2.0 which accompanies this
@@ -57,18 +57,18 @@ class ChatClient:
         self.write('Enter your name: ')
         name = (await self.readline()).rstrip('\n')
 
-        self.write('\n%d other users are connected.\n\n' % len(self._clients))
+        self.write(f'\n{len(self._clients)} other users are connected.\n\n')
 
         self._clients.append(self)
-        self.broadcast('*** %s has entered chat ***\n' % name)
+        self.broadcast(f'*** {name} has entered chat ***\n')
 
         try:
             async for line in self._process.stdin:
-                self.broadcast('%s: %s' % (name, line))
+                self.broadcast(f'{name}: {line}')
         except asyncssh.BreakReceived:
             pass
 
-        self.broadcast('*** %s has left chat ***\n' % name)
+        self.broadcast(f'*** {name} has left chat ***\n')
         self._clients.remove(self)
 
 async def start_server() -> None:
@@ -76,7 +76,7 @@ async def start_server() -> None:
                           authorized_client_keys='ssh_user_ca',
                           process_factory=ChatClient.handle_client)
 
-loop = asyncio.get_event_loop()
+loop = asyncio.new_event_loop()
 
 try:
     loop.run_until_complete(start_server())
