@@ -1,6 +1,6 @@
-#!/usr/bin/env python3.6
+#!/usr/bin/env python3.7
 #
-# Copyright (c) 2016-2021 by Ron Frederick <ronf@timeheart.net> and others.
+# Copyright (c) 2016-2024 by Ron Frederick <ronf@timeheart.net> and others.
 #
 # This program and the accompanying materials are made available under
 # the terms of the Eclipse Public License v2.0 which accompanies this
@@ -35,14 +35,14 @@ async def run_multiple_clients() -> None:
 
     for i, result in enumerate(results, 1):
         if isinstance(result, Exception):
-            print('Task %d failed: %s' % (i, str(result)))
+            print(f'Task {i} failed: {result}')
         elif result.exit_status != 0:
-            print('Task %d exited with status %s:' % (i, result.exit_status))
+            print(f'Task {i} exited with status {result.exit_status}:')
             print(result.stderr, end='')
         else:
-            print('Task %d succeeded:' % i)
+            print(f'Task {i} succeeded:')
             print(result.stdout, end='')
 
         print(75*'-')
 
-asyncio.get_event_loop().run_until_complete(run_multiple_clients())
+asyncio.run(run_multiple_clients())
