@@ -1,6 +1,6 @@
-#!/usr/bin/env python3.6
+#!/usr/bin/env python3.7
 #
-# Copyright (c) 2013-2021 by Ron Frederick <ronf@timeheart.net> and others.
+# Copyright (c) 2013-2024 by Ron Frederick <ronf@timeheart.net> and others.
 #
 # This program and the accompanying materials are made available under
 # the terms of the Eclipse Public License v2.0 which accompanies this
@@ -31,7 +31,7 @@ class MySSHTCPSession(asyncssh.SSHTCPSession):
 
 def connection_requested(orig_host: str,
                          orig_port: int) -> asyncssh.SSHTCPSession:
-    print('Connection received from %s, port %s' % (orig_host, orig_port))
+    print(f'Connection received from {orig_host}, port {orig_port}')
     return MySSHTCPSession()
 
 async def run_client() -> None:
@@ -45,6 +45,6 @@ async def run_client() -> None:
             print('Listener couldn\'t be opened.', file=sys.stderr)
 
 try:
-    asyncio.get_event_loop().run_until_complete(run_client())
+    asyncio.run(run_client())
 except (OSError, asyncssh.Error) as exc:
     sys.exit('SSH connection failed: ' + str(exc))
