@@ -1,6 +1,6 @@
-#!/usr/bin/env python3.6
+#!/usr/bin/env python3.7
 #
-# Copyright (c) 2013-2021 by Ron Frederick <ronf@timeheart.net> and others.
+# Copyright (c) 2013-2024 by Ron Frederick <ronf@timeheart.net> and others.
 #
 # This program and the accompanying materials are made available under
 # the terms of the Eclipse Public License v2.0 which accompanies this
@@ -41,11 +41,11 @@ async def handle_client(process: asyncssh.SSHServerProcess) -> None:
                 try:
                     total += int(line)
                 except ValueError:
-                    process.stderr.write('Invalid number: %s\n' % line)
+                    process.stderr.write(f'Invalid number: {line}\n')
     except asyncssh.BreakReceived:
         pass
 
-    process.stdout.write('Total = %s\n' % total)
+    process.stdout.write(f'Total = {total}\n')
     process.exit(0)
 
 async def start_server() -> None:
@@ -53,7 +53,7 @@ async def start_server() -> None:
                           authorized_client_keys='ssh_user_ca',
                           process_factory=handle_client)
 
-loop = asyncio.get_event_loop()
+loop = asyncio.new_event_loop()
 
 try:
     loop.run_until_complete(start_server())
