@@ -1,6 +1,6 @@
-#!/usr/bin/env python3.6
+#!/usr/bin/env python3.7
 #
-# Copyright (c) 2015-2021 by Ron Frederick <ronf@timeheart.net> and others.
+# Copyright (c) 2015-2024 by Ron Frederick <ronf@timeheart.net> and others.
 #
 # This program and the accompanying materials are made available under
 # the terms of the Eclipse Public License v2.0 which accompanies this
@@ -28,6 +28,6 @@ async def run_client() -> None:
             await sftp.get('example.txt')
 
 try:
-    asyncio.get_event_loop().run_until_complete(run_client())
+    asyncio.run(run_client())
 except (OSError, asyncssh.Error) as exc:
     sys.exit('SFTP operation failed: ' + str(exc))
