@@ -1,6 +1,6 @@
-#!/usr/bin/env python3.6
+#!/usr/bin/env python3.7
 #
-# Copyright (c) 2013-2021 by Ron Frederick <ronf@timeheart.net> and others.
+# Copyright (c) 2013-2024 by Ron Frederick <ronf@timeheart.net> and others.
 #
 # This program and the accompanying materials are made available under
 # the terms of the Eclipse Public License v2.0 which accompanies this
@@ -24,22 +24,22 @@
 # private key in it to use as a server host key. An SSH host certificate
 # can optionally be provided in the file ``ssh_host_key-cert.pub``.
 
-import asyncio, asyncssh, crypt, sys
+import asyncio, asyncssh, bcrypt, sys
 from typing import Optional
 
-passwords = {'guest': '',                 # guest account with no password
-             'user123': 'qV2iEadIGV2rw'   # password of 'secretpw'
+passwords = {'guest': b'',                # guest account with no password
+             'user123': bcrypt.hashpw(b'secretpw', bcrypt.gensalt()),
             }
 
 def handle_client(process: asyncssh.SSHServerProcess) -> None:
-    process.stdout.write('Welcome to my SSH server, %s!\n' %
-                         process.get_extra_info('username'))
+    username = process.get_extra_info('username')
+    process.stdout.write(f'Welcome to my SSH server, {username}!\n')
     process.exit(0)
 
 class MySSHServer(asyncssh.SSHServer):
     def connection_made(self, conn: asyncssh.SSHServerConnection) -> None:
-        print('SSH connection received from %s.' %
-                  conn.get_extra_info('peername')[0])
+        peername = conn.get_extra_info('peername')[0]
+        print(f'SSH connection received from {peername}.')
 
     def connection_lost(self, exc: Optional[Exception]) -> None:
         if exc:
@@ -49,21 +49,25 @@ class MySSHServer(asyncssh.SSHServer):
 
     def begin_auth(self, username: str) -> bool:
         # If the user's password is the empty string, no auth is required
-        return passwords.get(username) != ''
+        return passwords.get(username) != b''
 
     def password_auth_supported(self) -> bool:
         return True
 
     def validate_password(self, username: str, password: str) -> bool:
-        pw = passwords.get(username, '*')
-        return crypt.crypt(password, pw) == pw
+        if username not in passwords:
+            return False
+        pw = passwords[username]
+        if not password and not pw:
+            return True
+        return bcrypt.checkpw(password.encode('utf-8'), pw)
 
 async def start_server() -> None:
     await asyncssh.create_server(MySSHServer, '', 8022,
                                  server_host_keys=['ssh_host_key'],
                                  process_factory=handle_client)
 
-loop = asyncio.get_event_loop()
+loop = asyncio.new_event_loop()
 
 try:
     loop.run_until_complete(start_server())
