@@ -1,4 +1,4 @@
-# Copyright (c) 2016-2022 by Ron Frederick <ronf@timeheart.net> and others.
+# Copyright (c) 2016-2025 by Ron Frederick <ronf@timeheart.net> and others.
 #
 # This program and the accompanying materials are made available under
 # the terms of the Eclipse Public License v2.0 which accompanies this
@@ -75,6 +75,9 @@ class _NullServer(Server):
 
         return False
 
+    async def auth_completed(self):
+        """Handle client authentication request"""
+
 
 class _HostBasedServer(Server):
     """Server for testing host-based authentication"""
@@ -1132,7 +1135,7 @@ class _TestPublicKeyAuth(ServerTestCase):
             return 'passphrase'
 
         async with self.connect(username='ckey', client_keys='ckey_encrypted',
-                                passphrase=_passphrase):
+                                agent_path=None, passphrase=_passphrase):
             pass
 
     @asynctest
@@ -1144,7 +1147,7 @@ class _TestPublicKeyAuth(ServerTestCase):
             return 'passphrase'
 
         async with self.connect(username='ckey', client_keys='ckey_encrypted',
-                                passphrase=_passphrase):
+                                agent_path=None,passphrase=_passphrase):
             pass
 
     @asynctest
@@ -1157,7 +1160,7 @@ class _TestPublicKeyAuth(ServerTestCase):
 
         async with self.connect(username='ckey',
                                 client_keys=['ckey_encrypted'],
-                                passphrase=_passphrase):
+                                agent_path=None, passphrase=_passphrase):
             pass
 
     @asynctest
@@ -1170,7 +1173,7 @@ class _TestPublicKeyAuth(ServerTestCase):
 
         async with self.connect(username='ckey',
                                 client_keys=['ckey_encrypted'],
-                                passphrase=_passphrase):
+                                agent_path=None, passphrase=_passphrase):
             pass
 
     @asynctest
@@ -1179,7 +1182,7 @@ class _TestPublicKeyAuth(ServerTestCase):
 
         with self.assertRaises(asyncssh.KeyEncryptionError):
             await self.connect(username='ckey', client_keys='ckey_encrypted',
-                               passphrase='xxx')
+                               agent_path=None, passphrase='xxx')
 
     @asynctest
     async def test_encrypted_client_key_missing_passphrase(self):
